@@ -853,7 +853,7 @@ DEFINE_SET_FIELD(jint, Int)
 DEFINE_SET_FIELD(jlong, Long)
 DEFINE_SET_FIELD(jshort, Short)
 
-void JCCEnv::setClassPath(const char *classPath)
+bool JCCEnv::setClassPath(const char *classPath)
 {
     JNIEnv *vm_env = get_vm_env();
     jclass _ucl = (jclass) vm_env->FindClass("java/net/URLClassLoader");
@@ -861,6 +861,10 @@ void JCCEnv::setClassPath(const char *classPath)
     jmethodID mid = vm_env->GetStaticMethodID(_ucl, "getSystemClassLoader",
                                               "()Ljava/lang/ClassLoader;");
     jobject classLoader = vm_env->CallStaticObjectMethod(_ucl, mid);
+
+    if (!vm_env->IsInstanceOf(classLoader, _ucl))
+        return false;
+    
     jmethodID mf = vm_env->GetMethodID(_fil, "<init>", "(Ljava/lang/String;)V");
     jmethodID mu = vm_env->GetMethodID(_fil, "toURL", "()Ljava/net/URL;");
     jmethodID ma = vm_env->GetMethodID(_ucl, "addURL", "(Ljava/net/URL;)V");
@@ -882,6 +886,8 @@ void JCCEnv::setClassPath(const char *classPath)
         vm_env->CallVoidMethod(classLoader, ma, url);
     }
     free(path);
+
+    return true;
 }
 
 char *JCCEnv::getClassPath()
