@@ -33,6 +33,13 @@ class InvalidArgsError(Exception):
     pass
 
 
+class RequiredPackageError(Exception):
+
+    def __str__(self):
+        return "%s is required, please add a --package declaration for it, " \
+               "or its parameters, to the JCC invocation." %(self.args[0])
+
+
 _jcc3._set_exception_types(JavaError, InvalidArgsError)
 from ._jcc3 import findClass as _findClass
 from ._jcc3 import *
@@ -127,7 +134,8 @@ PRIMITIVES = { 'boolean': 'Z',
 RESERVED = set(['delete', 'and', 'or', 'not', 'xor', 'union', 'register',
                 'const', 'bool', 'operator', 'typeof', 'asm', 'mutable',
                 'inline', 'typedef', 'struct', 'extern',
-                'NULL', 'DOMAIN', 'IGNORE', 'min', 'max', 'PREFIX', 'NAN'])
+                'NULL', 'DOMAIN', 'IGNORE', 'min', 'max', 'PREFIX', 'NAN',
+                'DEBUG'])
 
 RENAME_METHOD_SUFFIX = '_'
 RENAME_FIELD_SUFFIX = '__'
@@ -439,8 +447,9 @@ def jcc(args):
     use_distutils = False
     shared = False
     dist = False
+    wheel = False
     wininst = False
-    find_jvm_dll = False
+    find_jvm_dll = "client" if sys.version_info >= (3, 8) else None
     compiler = None
     generics = hasattr(_jcc3, "Type")
     arch = []
@@ -546,6 +555,10 @@ def jcc(args):
             elif arg == '--bdist':
                 from .python import compile
                 dist = True
+            elif arg == '--wheel':
+                from .python import compile
+                wheel = True
+                dist = True
             elif arg == '--wininst':
                 from .python import compile
                 wininst = True
@@ -562,7 +575,8 @@ def jcc(args):
             elif arg == '--no-generics':
                 generics = False
             elif arg == '--find-jvm-dll':
-                find_jvm_dll = True
+                i += 1
+                find_jvm_dll = args[i]
             elif arg == '--resources':
                 i += 1
                 resources.append(args[i])
@@ -585,17 +599,20 @@ def jcc(args):
     initvm_args['maxstack'] = '512k'
     initvm_args['vmargs'] = vmargs
 
-    env = initVM(os.pathsep.join(classpath) or None, **initvm_args)
-
-    typeset = set()
-    excludes = set(excludes)
-
     if imports:
         if shared:
             imports = dict((__import__(import_), set()) for import_ in imports)
         else:
             raise ValueError("--shared must be used when using --import")
 
+        for import_ in imports.keys():
+            classpath.append(import_.CLASSPATH)
+
+    env = initVM(os.pathsep.join(classpath) or None, **initvm_args)
+
+    typeset = set()
+    excludes = set(excludes)
+
     if recompile or not build and (install or dist or egg_info):
         if moduleName is None:
             raise ValueError('module name not specified (use --python)')
@@ -603,24 +620,21 @@ def jcc(args):
             compile(env, os.path.dirname(args[0]), output, moduleName,
                     install, dist, debug, jars, version,
                     prefix, root, install_dir, home_dir, use_distutils,
-                    shared, compiler, modules, wininst, find_jvm_dll,
+                    shared, compiler, modules, wheel, wininst, find_jvm_dll,
                     arch, generics, resources, imports, use_full_names,
                     egg_info, extra_setup_args)
     else:
         if imports:
-            def walk(args, dirname, names):
-                (include, importset) = args
-                for name in names:
-                    if name.endswith('.h'):
-                        className = os.path.join(dirname[len(include) + 1:],
-                                                 name[:-2])
-                        if os.path.sep != '/':
-                            className = className.replace(os.path.sep, '/')
-                        importset.add(findClass(className))
             for import_, importset in imports.items():
-                env._addClassPath(import_.CLASSPATH)
                 include = os.path.join(import_.__module_dir__, 'include')
-                os.path.walk(include, walk, (include, importset))
+                for dirpath, dirnames, filenames in os.walk(include):
+                    for name in filenames:
+                        if name.endswith('.h'):
+                            className = os.path.join(dirpath[len(include) + 1:],
+                                                     name[:-2])
+                            if os.path.sep != '/':
+                                className = className.replace(os.path.sep, '/')
+                            importset.add(findClass(className))
                 typeset.update(importset)
         typeset.add(findClass('java/lang/Object'))
         typeset.add(findClass('java/lang/Class'))
@@ -762,7 +776,7 @@ def jcc(args):
                 compile(env, os.path.dirname(args[0]), output, moduleName,
                         install, dist, debug, jars, version,
                         prefix, root, install_dir, home_dir, use_distutils,
-                        shared, compiler, modules, wininst, find_jvm_dll,
+                        shared, compiler, modules, wheel, wininst, find_jvm_dll,
                         arch, generics, resources, imports, use_full_names,
                         egg_info, extra_setup_args)
 
@@ -792,6 +806,9 @@ def header(env, out, cls, typeset, packages, excludes, generics,
                 interfaces.append(interface)
                 if pt is not None:
                     for ta in pt.getActualTypeArguments():
+                        if not known(ta, typeset, declares, packages,
+                                     excludes, True):
+                            raise RequiredPackageError(ta)
                         addRequiredTypes(ta, typeset, True)
     else:
         for interface in cls.getInterfaces():
@@ -812,8 +829,10 @@ def header(env, out, cls, typeset, packages, excludes, generics,
         if generics:
             for clsParam in getTypeParameters(cls):
                 if Class.instance_(clsParam):
+                    if not known(clsParam, typeset, declares, packages,
+                                 excludes, True):
+                        raise RequiredPackageError(clsParam)
                     addRequiredTypes(clsParam, typeset, True)
-                    known(clsParam, typeset, declares, packages, excludes, True)
     else:
         superClsNames = ['JObject']
 
