@@ -1590,8 +1590,9 @@ def module(out, allInOne, classes, imports, cppdir, moduleName,
 
 def compile(env, jccPath, output, moduleName, install, dist, debug, jars,
             version, prefix, root, install_dir, home_dir, use_distutils,
-            shared, compiler, modules, wininst, find_jvm_dll, arch, generics,
-            resources, imports, use_full_names, egg_info, extra_setup_args):
+            shared, compiler, modules, wheel, wininst, find_jvm_dll, arch,
+            generics, resources, imports, use_full_names, egg_info,
+            extra_setup_args):
     try:
         if use_distutils:
             raise ImportError
@@ -1606,25 +1607,26 @@ def compile(env, jccPath, output, moduleName, install, dist, debug, jars,
         with_setuptools = False
 
     extname = '_%s' %(moduleName)
-
+    if not os.path.isdir(os.path.join(output, extname)):
+        raise FileNotFoundError(
+            "%s directory not found, module %s needs to be compiled first" %(
+                os.path.join(output, extname), extname))
+    
     modulePath = os.path.join(output, moduleName)
     if not os.path.isdir(modulePath):
         os.makedirs(modulePath)
 
     out = open(os.path.join(modulePath, '__init__.py'), 'w')
     line(out)
-    if shared:
-        line(out, 0, "import os, sys")
-        line(out)
+    line(out, 0, "import os")
+    if find_jvm_dll:
+        line(out, 0, "import sys")
         line(out, 0, "if sys.platform == 'win32':")
-        if find_jvm_dll:
-            line(out, 1, "from jcc.windows import add_jvm_dll_directory_to_path")
-            line(out, 1, "add_jvm_dll_directory_to_path()")
-        line(out, 1, "import jcc")
-        line(out, 0, "from . import %s", extname)
-    else:
-        line(out, 0, "import os")
-        line(out, 0, "from . import %s", extname)
+        line(out, 1, "from .windows import add_jvm_dll_directory_to_path")
+        line(out, 1, "add_jvm_dll_directory_to_path('%s')" %(find_jvm_dll))
+        modules.append(os.path.join(
+            os.path.dirname(os.path.realpath(__file__)), 'windows.py'))
+    line(out, 0, "from . import %s", extname)
     line(out)
     line(out, 0, '__module_dir__ = os.path.abspath(os.path.dirname(__file__))')
 
@@ -1801,7 +1803,7 @@ def compile(env, jccPath, output, moduleName, install, dist, debug, jars,
         if wininst:
             script_args.append('bdist_wininst')
         elif with_setuptools:
-            script_args.append('bdist_egg')
+            script_args.append('bdist_wheel' if wheel else 'bdist_egg')
         else:
             script_args.append('bdist')
 
@@ -1869,6 +1871,7 @@ def compile(env, jccPath, output, moduleName, install, dist, debug, jars,
                 i += 1
         config_vars['CFLAGS'] = ' '.join(cflags)
 
+    print("extension args = %s" % args)
     extensions = [Extension('.'.join([moduleName, extname]), **args)]
     script_args.extend(extra_setup_args)
 
