@@ -730,6 +730,41 @@ template<> class JArray<jchar> : public java::lang::Object {
   public:
     Py_ssize_t length;
 
+    static jsize jchar_length(PyObject *sequence)
+    {
+        if (PyUnicode_Check(sequence))
+        {
+            PyUnicode_READY(sequence);
+            
+            switch (PyUnicode_KIND(sequence)) {
+              case PyUnicode_1BYTE_KIND:
+              case PyUnicode_2BYTE_KIND:
+                return (jsize) PyUnicode_GET_LENGTH(sequence);
+
+              case PyUnicode_4BYTE_KIND: {
+                jsize len = (jsize) PyUnicode_GET_LENGTH(sequence);
+                Py_UCS4 *pchars = PyUnicode_4BYTE_DATA(sequence);
+                jsize count = 0;
+
+                for (jsize i = 0; i < len; ++i) {
+                  Py_UCS4 c = pchars[i];
+
+                  if (c <= 0xd7ff || (c >= 0xe000 && c <= 0xffff)) {
+                    count += 1;
+                  } else if (c >= 0x10000 && c <= 0x10ffff) {
+                    count += 2;
+                  } else if (c >= 0xd800 && c <= 0xdfff) {
+                    count += 1;
+                  }
+                }
+                return count;
+              }
+            }
+        }
+
+        return PySequence_Length(sequence);
+    }
+
     class arrayElements {
     private:
         jboolean isCopy;
@@ -765,21 +800,53 @@ template<> class JArray<jchar> : public java::lang::Object {
     }
 
 #ifdef PYTHON
-    JArray<jchar>(PyObject *sequence) : java::lang::Object(env->get_vm_env()->NewCharArray(PySequence_Length(sequence))) {
+    JArray<jchar>(PyObject *sequence) : java::lang::Object(env->get_vm_env()->NewCharArray(jchar_length(sequence))) {
         length = env->getArrayLength((jarray) this$);
         arrayElements elts = elements();
         jchar *buf = (jchar *) elts;
 
         if (PyUnicode_Check(sequence))
         {
-            if (sizeof(Py_UNICODE) == sizeof(jchar))
-                memcpy(buf, PyUnicode_AS_UNICODE(sequence),
-                       length * sizeof(jchar));
-            else
-            {
-                Py_UNICODE *pchars = PyUnicode_AS_UNICODE(sequence);
-                for (Py_ssize_t i = 0; i < length; i++)
+            switch (PyUnicode_KIND(sequence)) {
+              case PyUnicode_1BYTE_KIND: {
+                jsize len = (jsize) PyUnicode_GET_LENGTH(sequence);
+                Py_UCS1 *pchars = PyUnicode_1BYTE_DATA(sequence);
+
+                for (jsize i = 0; i < len; i++)
                     buf[i] = (jchar) pchars[i];
+                break;
+              }
+                
+              case PyUnicode_2BYTE_KIND: {
+                jsize len = (jsize) PyUnicode_GET_LENGTH(sequence);
+                Py_UCS2 *pchars = PyUnicode_2BYTE_DATA(sequence);
+
+                memcpy(buf, pchars, len * sizeof(jchar));
+                break;
+              }
+
+              case PyUnicode_4BYTE_KIND: {
+// borrowed from ICU (http://icu-project.org/apiref/icu4c/utf16_8h.html)
+#define U16_LEAD(c) (jchar) (((c) >> 10) + 0xd7c0)
+#define U16_TRAIL(c) (jchar) (((c) & 0x3ff) | 0xdc00)
+                jsize len = (jsize) PyUnicode_GET_LENGTH(sequence);
+                Py_UCS4 *pchars = PyUnicode_4BYTE_DATA(sequence);
+                jsize pos = 0;
+
+                for (jsize i = 0; i < len; ++i) {
+                  Py_UCS4 c = pchars[i];
+
+                  if (c <= 0xd7ff || (c >= 0xe000 && c <= 0xffff)) {
+                    buf[pos++] = (jchar) c;
+                  } else if (c >= 0x10000 && c <= 0x10ffff) {
+                    buf[pos++] = (jchar) U16_LEAD(c);
+                    buf[pos++] = (jchar) U16_TRAIL(c);
+                  } else if (c >= 0xd800 && c <= 0xdfff) {
+                    buf[pos++] = (jchar) 0xfffd;  // invalid char
+                  }
+                }
+                break;
+              }
             }
         }
         else
@@ -791,7 +858,7 @@ template<> class JArray<jchar> : public java::lang::Object {
 
                 if (PyUnicode_Check(obj) && (PyUnicode_GET_LENGTH(obj) == 1))
                 {
-                    buf[i] = (jchar) PyUnicode_AS_UNICODE(obj)[0];
+                    buf[i] = (jchar) PyUnicode_READ_CHAR(obj, 0);
                     Py_DECREF(obj);
                 }
                 else
@@ -814,7 +881,7 @@ template<> class JArray<jchar> : public java::lang::Object {
                 break;
 
             if (PyUnicode_Check(obj) && (PyUnicode_GET_LENGTH(obj) == 1))
-                buf[i] = (jchar) PyUnicode_AS_UNICODE(obj)[0];
+                buf[i] = (jchar) PyUnicode_READ_CHAR(obj, 0);
             else
             {
                 PyErr_SetObject(PyExc_TypeError, obj);
@@ -844,19 +911,7 @@ template<> class JArray<jchar> : public java::lang::Object {
         arrayElements elts = elements();
         jchar *buf = (jchar *) elts;
 
-        if (sizeof(Py_UNICODE) == sizeof(jchar))
-            return PyUnicode_FromUnicode((const Py_UNICODE *) buf + lo,
-                                         hi - lo);
-        else
-        {
-            PyObject *string = PyUnicode_FromUnicode(NULL, hi - lo);
-            Py_UNICODE *pchars = PyUnicode_AS_UNICODE(string);
-
-            for (Py_ssize_t i = lo; i < hi; i++)
-                pchars[i - lo] = (Py_UNICODE) buf[i];
-
-            return string;
-        }
+        return env->fromJChars(buf + lo, hi - lo);
     }
 
     PyObject *get(Py_ssize_t n)
@@ -869,18 +924,7 @@ template<> class JArray<jchar> : public java::lang::Object {
             if (n >= 0 && n < length)
             {
                 jchar c = (*this)[n];
-
-                if (sizeof(Py_UNICODE) == sizeof(jchar))
-                    return PyUnicode_FromUnicode((const Py_UNICODE *) &c, 1);
-                else
-                {
-                    PyObject *string = PyUnicode_FromUnicode(NULL, 1);
-                    Py_UNICODE *pchars = PyUnicode_AS_UNICODE(string);
-
-                    pchars[0] = (Py_UNICODE) c;
-
-                    return string;
-                }
+                return env->fromJChars(&c, 1);
             }
         }
 
@@ -908,7 +952,7 @@ template<> class JArray<jchar> : public java::lang::Object {
                     return -1;
                 }
 
-                elements()[n] = (jchar) PyUnicode_AS_UNICODE(obj)[0];
+                elements()[n] = (jchar) PyUnicode_READ_CHAR(obj, 0);
                 return 0;
             }
         }
