@@ -1708,14 +1708,24 @@ PyTypeObject *makeType(PyType_Def *def)
             count += 1;
 
         PyObject *bases = PyTuple_New(count);
+        int basicsize = def->spec.basicsize;
 
-        for (int i = 0; i < count; ++i)
+        for (int i = 0; i < count; ++i) {
             PyTuple_SET_ITEM(bases, i, (PyObject *) makeType(def->bases[i]));
+            if (basicsize < def->bases[i]->spec.basicsize)
+                basicsize = def->bases[i]->spec.basicsize;
+        }
 
+        def->spec.basicsize = basicsize;  // or typeobject.c, line 2092, fails
         def->type = (PyTypeObject *)
             PyType_FromSpecWithBases(&def->spec, bases);
+
         Py_DECREF(bases);
     }
+    else
+    {
+        Py_INCREF((PyObject *) def->type);
+    }
 
     return def->type;
 }
@@ -1726,14 +1736,25 @@ void installType(PyTypeObject **type, PyType_Def *def,
     if (*type == NULL)
     {
         *type = makeType(def);
-        Py_INCREF(*type);
 
         if (isExtension)
         {
+#if PY_VERSION_HEX >= 0x030b0000
+            Py_SET_TYPE(*type, PY_TYPE(FinalizerClass));
+#else
             Py_TYPE(*type) = PY_TYPE(FinalizerClass);
+#endif
             Py_INCREF(PY_TYPE(FinalizerClass));
         }
 
+        PyObject *module_name = PyModule_GetNameObject(module);
+
+        if (module_name != NULL)
+        {
+            PyDict_SetItemString((*type)->tp_dict, "__module__", module_name);
+            Py_DECREF(module_name);
+        }
+
         PyModule_AddObject(module, name, (PyObject *) *type);
     }
 }
